@@ -15,7 +15,7 @@ from numba.core.errors import NumbaPerformanceWarning
 
 warnings.filterwarnings("ignore", category=NumbaPerformanceWarning)
 
-__version__ = "0.7.4"
+__version__ = "0.7.5"
 
 """
 Copyright 2018, Alexander J. Tarashansky, All rights reserved.
@@ -882,6 +882,7 @@ class SAM(object):
         sparse_pca=False,
         proj_kwargs={},
         project_weighted=True,
+        seed = 0,
         weight_mode='dispersion'
     ):
         """Runs the Self-Assembling Manifold algorithm.
@@ -975,6 +976,7 @@ class SAM(object):
             "sparse_pca": sparse_pca,
             "project_weighted": project_weighted,
             "weight_mode": weight_mode,
+            "seed": seed,
         }
 
         numcells = D.shape[0]
@@ -1002,7 +1004,7 @@ class SAM(object):
             npcs = 500
 
         tinit = time.time()
-
+        np.random.seed(seed)
         edm = sp.coo_matrix((numcells, numcells), dtype="i").tolil()
         nums = np.arange(edm.shape[1])
         RINDS = np.random.randint(0, numcells, (k - 1) * numcells).reshape(
@@ -1042,7 +1044,7 @@ class SAM(object):
             old = new
 
             W, wPCA_data, EDM, = self.calculate_nnm(
-                n_genes, preprocessing, npcs, nnas, weight_PCs, sparse_pca,project_weighted=project_weighted,weight_mode=weight_mode
+                n_genes, preprocessing, npcs, nnas, weight_PCs, sparse_pca,project_weighted=project_weighted,weight_mode=weight_mode,seed=seed
             )
             new = W
             err = ((new - old) ** 2).mean() ** 0.5
@@ -1060,7 +1062,7 @@ class SAM(object):
         elif projection == "umap":
             if verbose:
                 print("Computing the UMAP embedding...")
-            self.run_umap(**proj_kwargs)
+            self.run_umap(seed=seed,**proj_kwargs)
         elif projection == "diff_umap":
             if verbose:
                 print("Computing the diffusion UMAP embedding...")
@@ -1073,7 +1075,7 @@ class SAM(object):
 
     def calculate_nnm(
         self, n_genes, preprocessing, npcs, num_norm_avg, weight_PCs, sparse_pca,
-        update_manifold=True,project_weighted=True,weight_mode='dispersion',
+        update_manifold=True,project_weighted=True,weight_mode='dispersion',seed=0
     ):
         numcells = self.adata.shape[0]
         D = self.adata.X
@@ -1122,14 +1124,14 @@ class SAM(object):
                     D_sub,
                     npcs=npcs,
                     do_weight=weight_PCs,
-                    solver="auto",
+                    solver="auto",seed=seed
                 )
             else:
                 g_weighted, pca = ut.weighted_PCA(
                     D_sub,
                     npcs=npcs,
                     do_weight=weight_PCs,
-                    solver="full",
+                    solver="full",seed=seed
                 )
             self.pca_obj = pca
             self.components = pca.components_
@@ -1142,7 +1144,7 @@ class SAM(object):
                     g_weighted = g_weighted * (ev ** 0.5)
         else:
             npcs=min(npcs, min((D.shape[0],gkeep.size)) - 1)
-            output = ut._pca_with_sparse(D_sub, npcs)
+            output = ut._pca_with_sparse(D_sub, npcs,seed = seed)
             self.components = output['components']
             g_weighted = output['X_pca']
             if not project_weighted:
@@ -1204,7 +1206,7 @@ class SAM(object):
             tsne2d = dt
             self.adata.obsm["X_tsne"] = tsne2d
 
-    def run_umap(self, X="X_pca", metric=None, **kwargs):
+    def run_umap(self, X="X_pca", metric=None,seed = 0, **kwargs):
         """Wrapper for umap-learn.
 
         See https://github.com/lmcinnes/umap sklearn for the documentation
@@ -1222,12 +1224,12 @@ class SAM(object):
             else:
                 X = self.adata.obsm[X]
             # print(X.shape)
-            umap_obj = umap.UMAP(metric=metric, **kwargs)
+            umap_obj = umap.UMAP(metric=metric,random_state=seed, **kwargs)
             umap2d = umap_obj.fit_transform(X)
             self.adata.obsm["X_umap"] = umap2d
             self.umap_obj = umap_obj
         else:
-            umap_obj = umap.UMAP(metric=metric, **kwargs)
+            umap_obj = umap.UMAP(metric=metric,random_state=seed, **kwargs)
             dt = umap_obj.fit_transform(X)
             return dt, umap_obj
 
